@@ -1,5 +1,5 @@
 package Win32::Exe;
-$Win32::Exe::VERSION = '0.11';
+$Win32::Exe::VERSION = '0.13';
 
 =head1 NAME
 
@@ -7,8 +7,8 @@ Win32::Exe - Manipulate Win32 executable files
 
 =head1 VERSION
 
-This document describes version 0.10 of Win32::Exe, released
-January 14, 2007.
+This document describes version 0.13 of Win32::Exe, released
+April 28, 2010.
 
 =head1 SYNOPSIS
 
@@ -17,12 +17,12 @@ January 14, 2007.
 
     # Get version information
     print $exe->version_info->get('FileDescription'), ": ",
-	$exe->version_info->get('LegalCopyright'), "\n";
+    $exe->version_info->get('LegalCopyright'), "\n";
 
     # Dump icons in an executable
     foreach my $icon ($exe->icons) {
-	printf "Icon: %s x %s (%s colors)\n",
-	       $icon->Width, $icon->Height, 2 ** $icon->BitCount;
+    printf "Icon: %s x %s (%s colors)\n",
+           $icon->Width, $icon->Height, 2 ** $icon->BitCount;
     }
 
     # Import icons from a .exe or .ico file and writes back the file
@@ -37,35 +37,38 @@ January 14, 2007.
     # or a default
     $exe->update( defaultmanifest => 1 );
     
+    # Get manifest object
+    $manifest = $exe->get_manifest if $exe->has_manifest 
+    
 
 =head1 DESCRIPTION
 
 This module parses and manipulating Win32 PE/COFF executable headers,
-including version information, icons and other resources.
+including version information, icons, manifest and other resources.
+The module Win32::Exe::Manifest can be used for manifest handling.
 
-More documentation will be provided in due time.  For now, please look
-at the test files in the source distributions F<t/> directory for examples
-of using this module.
+Also, please see the test files in the source distributions
+F<t/> directory for examples of using this module.
 
 =cut
 
 use strict;
 use base 'Win32::Exe::Base';
 use constant FORMAT => (
-    Magic	    => 'a2',    # "MZ"
-    _		    => 'a58',
-    PosPE	    => 'V',
-    _		    => 'a{($PosPE > 64) ? $PosPE - 64 : "*"}',
-    PESig	    => 'a4',
-    Data	    => 'a*',
+    Magic       => 'a2',    # "MZ"
+    _           => 'a58',
+    PosPE       => 'V',
+    _           => 'a{($PosPE > 64) ? $PosPE - 64 : "*"}',
+    PESig       => 'a4',
+    Data        => 'a*',
 );
 use constant DELEGATE_SUBS => (
-    'IconFile'	=> [ 'dump_iconfile', 'write_iconfile' ],
+    'IconFile'  => [ 'dump_iconfile', 'write_iconfile' ],
 );
 use constant DISPATCH_FIELD => 'PESig';
 use constant DISPATCH_TABLE => (
-    "PE\0\0"	=> "PE",
-    '*'		=> sub { die "Incorrect PE header -- not a valid .exe file" },
+    "PE\0\0"    => "PE",
+    '*'     => sub { die "Incorrect PE header -- not a valid .exe file" },
 );
 use constant DEBUG_INDEX         => 6;
 use constant DEBUG_ENTRY_SIZE    => 28;
@@ -73,6 +76,11 @@ use constant DEBUG_ENTRY_SIZE    => 28;
 use File::Basename ();
 use Win32::Exe::IconFile;
 use Win32::Exe::DebugTable;
+use Win32::Exe::Manifest;
+
+sub is_application { ( $_[0]->is_assembly ) ? 0 : 1; }
+
+sub is_assembly { $_[0]->Characteristics & 0x2000; }
 
 sub resource_section {
     my ($self) = @_;
@@ -94,7 +102,7 @@ sub update_debug_directory {
     my ($self, $boundary, $extra) = @_;
 
     $self->SetSymbolTable( $self->SymbolTable + $extra )
-	if ($boundary <= $self->SymbolTable);
+    if ($boundary <= $self->SymbolTable);
 
     my @dirs = $self->data_directories;
     return if DEBUG_INDEX > $#dirs;
@@ -110,29 +118,29 @@ sub update_debug_directory {
     (($size % DEBUG_ENTRY_SIZE) == 0) or return;
 
     foreach my $section ($self->sections) {
-	my $offset = $section->FileOffset;
-	my $f_size = $section->FileSize;
-	my $v_addr = $section->VirtualAddress;
+    my $offset = $section->FileOffset;
+    my $f_size = $section->FileSize;
+    my $v_addr = $section->VirtualAddress;
 
-	next unless $v_addr <= $addr;
-	next unless $addr < ($v_addr + $f_size);
-	next unless ($addr + $size) < ($v_addr + $f_size);
+    next unless $v_addr <= $addr;
+    next unless $addr < ($v_addr + $f_size);
+    next unless ($addr + $size) < ($v_addr + $f_size);
 
-	$offset += $addr - $v_addr;
-	my $data = $self->substr($offset, $size);
+    $offset += $addr - $v_addr;
+    my $data = $self->substr($offset, $size);
 
-	my $table = Win32::Exe::DebugTable->new(\$data);
+    my $table = Win32::Exe::DebugTable->new(\$data);
 
-	foreach my $dir ($table->members) {
-	    next unless $boundary <= $dir->Offset;
+    foreach my $dir ($table->members) {
+        next unless $boundary <= $dir->Offset;
 
-	    $dir->SetOffset($dir->Offset + $extra);
-	    $dir->SetVirtualAddress($dir->VirtualAddress + $extra)
-		if $dir->VirtualAddress > 0;
-	}
+        $dir->SetOffset($dir->Offset + $extra);
+        $dir->SetVirtualAddress($dir->VirtualAddress + $extra)
+        if $dir->VirtualAddress > 0;
+    }
 
-	$self->substr($offset, $size, $table->dump);
-	last;
+    $self->substr($offset, $size, $table->dump);
+    last;
     }
 }
 
@@ -142,15 +150,15 @@ sub default_info {
     my $filename = File::Basename::basename($self->filename);
 
     return join(';',
-	"CompanyName=",
-	"FileDescription=",
-	"FileVersion=0.0.0.0",
-	"InternalName=$filename",
-	"LegalCopyright=",
-	"LegalTrademarks=",
-	"OriginalFilename=$filename",
-	"ProductName=",
-	"ProductVersion=0.0.0.0",
+    "CompanyName=",
+    "FileDescription=",
+    "FileVersion=0.0.0.0",
+    "InternalName=$filename",
+    "LegalCopyright=",
+    "LegalTrademarks=",
+    "OriginalFilename=$filename",
+    "ProductName=",
+    "ProductVersion=0.0.0.0",
     );
 }
 
@@ -166,37 +174,37 @@ sub update {
     }
 
     if (my $icon = $args{icon}) {
-	my @icons = Win32::Exe::IconFile->new($icon)->icons;
-	$self->set_icons(\@icons) if @icons;
+    my @icons = Win32::Exe::IconFile->new($icon)->icons;
+    $self->set_icons(\@icons) if @icons;
     }
 
     if (my $info = $args{info}) {
-	my @info = ($self->default_info, @$info);
+    my @info = ($self->default_info, @$info);
 
-	my @pairs;
-	foreach my $pairs (map split(/\s*;\s*(?=[\w\\\/]+\s*=)/, $_), @info) {
-	    my ($key, $val) = split(/\s*=\s*/, $pairs, 2);
-	    next if $key =~ /language/i;
+    my @pairs;
+    foreach my $pairs (map split(/\s*;\s*(?=[\w\\\/]+\s*=)/, $_), @info) {
+        my ($key, $val) = split(/\s*=\s*/, $pairs, 2);
+        next if $key =~ /language/i;
 
-	    if ($key =~ /^(product|file)version$/i) {
-		$key = "\u$1Version";
-		$val =~ /^(?:\d+\.)+\d+$/ or die "$key cannot be '$val'";
-		$val .= '.0' while $val =~ y/.,// < 3;
+        if ($key =~ /^(product|file)version$/i) {
+        $key = "\u$1Version";
+        $val =~ /^(?:\d+\.)+\d+$/ or die "$key cannot be '$val'";
+        $val .= '.0' while $val =~ y/.,// < 3;
 
-		push(@pairs,
-		    [ $key => $val ],
-		    [ "/StringFileInfo/#1/$key", $val ]);
-	    }
-	    else {
-		push(@pairs, [ $key => $val ]);
-	    }
-	}
+        push(@pairs,
+            [ $key => $val ],
+            [ "/StringFileInfo/#1/$key", $val ]);
+        }
+        else {
+        push(@pairs, [ $key => $val ]);
+        }
+    }
 
-	$self->set_version_info(\@pairs) if @pairs;
+    $self->set_version_info(\@pairs) if @pairs;
     }
 
     die "'gui' and 'console' cannot both be true"
-	if $args{gui} and $args{console};
+    if $args{gui} and $args{console};
 
     $self->SetSubsystem("windows") if $args{gui};
     $self->SetSubsystem("console") if $args{console};
@@ -215,7 +223,7 @@ sub set_icons {
 
     my $rsrc = $self->resource_section;
     my $name = eval { $rsrc->first_object('GroupIcon')->PathName }
-	    || '/#RT_GROUP_ICON/#1/#0';
+        || '/#RT_GROUP_ICON/#1/#0';
 
     $rsrc->remove('/#RT_GROUP_ICON');
     $rsrc->remove('/#RT_ICON');
@@ -256,11 +264,48 @@ sub manifest {
     }
 }    
 
+sub has_manifest {
+    my ($self) = @_;
+    my $rsrc = $self->resource_section or return 0;
+    if( my $obj = $rsrc->first_object('Manifest') ) {
+        return 1;
+    } else {
+        return 0;
+    }
+}
+
 sub set_manifest {
-    my ($self, $xml) = @_;
+    my ($self, $input) = @_;
+    # support code that passes xml, filepaths and objects
+    my $resid = 0;
+    my $xml;
+    if(ref($input) && $input->isa('Win32::Exe::Manifest')) {
+        $resid = $input->get_resource_id;
+        $xml = $input->output ;
+    } else {
+        my $filecontent;
+        eval {
+            my $paramisfile = 0;
+            {
+                no warnings qw( io );
+                $paramisfile = (-f $input);
+            }
+            if($paramisfile) {
+                open my $fh, '<', $input;
+                $filecontent = do { local $/; <$fh> };
+                my $errors = $@;
+                close($fh);
+                die $errors if $errors;
+            } else {
+                $filecontent = $input;
+            }
+        };
+        $xml = ( $@ ) ? $input : $filecontent;
+    }
+    $resid ||= 1;
     my $rsrc = $self->resource_section;
-    my $name = eval { $rsrc->first_object('Manifest')->PathName } || '/#RT_MANIFEST/#1/#0';
-    $rsrc->remove( $name  );
+    my $name = '/#RT_MANIFEST/#' . $resid . '/#0';
+    $rsrc->remove("/#RT_MANIFEST");
     my $manifest = $self->require_class('Resource::Manifest')->new;
     $manifest->SetPathName( $name ); 
     $manifest->set_parent( $rsrc );
@@ -269,11 +314,20 @@ sub set_manifest {
     $rsrc->refresh;
 }
 
+sub get_manifest {
+    my ($self) = @_;
+    my $mtype = ($self->is_assembly) ? 'assembly' : 'application';
+    my $mfestxml = $self->manifest->get_manifest;
+    my $mfest = Win32::Exe::Manifest->new($mfestxml, $mtype);
+    $mfest->set_resource_id( $self->manifest->get_manifest_id );
+    return $mfest;
+}
+
 sub add_default_manifest {
     my ($self) = @_;
     my $rsrc = $self->resource_section;
-    my $name = eval { $rsrc->first_object('Manifest')->PathName } || '/#RT_MANIFEST/#1/#0';
-    $rsrc->remove( $name  );
+    my $name = '/#RT_MANIFEST/#1/#0';
+    $rsrc->remove("/#RT_MANIFEST");
     my $manifest = $self->require_class('Resource::Manifest')->new;
     my $xml = $manifest->default_manifest;
     $manifest->SetPathName( $name ); 
@@ -281,7 +335,14 @@ sub add_default_manifest {
     $manifest->update_manifest( $xml );
     $rsrc->insert($manifest->PathName, $manifest);
     $rsrc->refresh;
-    $self->write;
+}
+
+sub merge_manifest {
+    my ($self, $mnf) = @_;
+    return if !(ref($mnf) && $mnf->isa('Win32::Exe::Manifest'));
+    my $main = $self->get_manifest;
+    $main->merge_manifest($mnf);
+    $self->set_manifest($main);
 }
 
 1;
@@ -292,13 +353,13 @@ __END__
 
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
-Mark Dootson
+Mark Dootson E<lt>mdootson@cpan.orgE<gt>
 
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004-2007 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2004-2007, 2010 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 
 This program is free software; you can redistribute it and/or 
 modify it under the same terms as Perl itself.
