@@ -1,6 +1,5 @@
 #!/usr/local/bin/perl
-# $File: //member/autrijus/Win32-Exe/script/exe_update.pl $ $Author: autrijus $
-# $Revision: #1 $ $Change: 9927 $ $DateTime: 2004/02/06 19:31:24 $
+# Copyright 2004, 2006, 2010 by Audrey Tang <cpan@audreyt.org>
 
 use strict;
 use File::Basename;
@@ -14,13 +13,14 @@ exe_update.pl - Modify windows executable files
 =head1 SYNOPSIS
 
 B<exe_update.pl> S<[ B<--gui> | B<--console> ]> S<[ B<--icon> I<iconfile> ]>
+              S<[ B<--manifest> I<manifestfile> ]>
               S<[ B<--info> I<key=value;...> ]> I<exefile>
 
 =head1 DESCRIPTION
 
 This program rewrites PE headers in a Windows executable file.  It can
 change whether the executable runs with a console window, as well as
-setting the icons and version information associated with it.
+setting the icons, manifest and version information associated with it.
 
 =head1 OPTIONS
 
@@ -46,6 +46,11 @@ Set the executable so it does not have a console window.
 Specify an icon file (in F<.ico>, F<.exe> or F<.dll> format) for the
 executable.
 
+=item B<-m>, B<--manifest>=I<FILE>
+
+Specify a manifest file in F<.xml> format for the
+executable.
+
 =item B<-N>, B<--info>=I<KEY=VAL>
 
 Attach version information for the executable.  The name/value pair is
@@ -67,17 +72,19 @@ Getopt::Long::GetOptions( $Options,
     'g|gui',            # No console window
     'c|console',        # Use console window
     'i|icon:s',         # Icon file
+    'm|manifest:s',     # manifest file
     'N|info:s@',        # Executable header info
 );
 
 my $exe = shift or die "Usage: " . basename($0) .
-    " [--gui | --console] [--icon file.ico] [--info key=value] file.exe\n";
+    " [--gui | --console] [--icon file.ico] [--manifest file.xml] [--info key=value] file.exe\n";
 
 Win32::Exe->new($exe)->update(
-    gui	    => $Options->{g},
-    console => $Options->{c},
-    icon    => $Options->{i},
-    info    => $Options->{N},
+    gui	     => $Options->{g},
+    console  => $Options->{c},
+    icon     => $Options->{i},
+    info     => $Options->{N},
+    manifest => $Options->{m},
 ) or die "Update of $exe failed!\n";
 
 __END__
@@ -88,7 +95,7 @@ Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004, 2006 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2004, 2006, 2010 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
